import numpy as np
from numpy import linalg as LA
from scipy.optimize import fsolve
import matplotlib.pyplot as plt

def heun_euler_one_step (f, t, y, h):
    k1 = f(t, y); k2 = f(t + h, y + h * k1); ynp1 = y + h * 0.5 * (k1 + k2)
    znp1 = y + h * k1; err = ynp1 - znp1
    return (ynp1, err)

def compute_h (err, hn, order, tolerance):
    if LA.norm(err, 2) <= (tolerance / pow(2.0, order + 1)):
        return 2 * hn
    else:
        return hn

def solve (f, t0, y0, tend, tolerance):
    t = t0; yn = y0; hn = 0.5; y = [y0]; time = [t0]; h = [hn]
    while t <= tend:
        (yn, err) = heun_euler_one_step (f, t, yn, hn)
        if LA.norm(err, 2) <= tolerance:
            y.append(yn); t = t + hn; time.append(t)
            hn = compute_h (err, hn, 1, tolerance); h.append(hn)
        else:
            hn = hn / 2.0
    return [ time, y, h]

def dynamic (t, y):
    return np.array([-y[1], y[0]])

[t, y, h] = solve (dynamic, 0.0, np.array([1., 0.]), 2*np.pi*10, 1e-2)


# def dynamic_stiff (t, y):
#     mu = 5.0
#     return np.array([y[1], mu * (1.0 - pow(y[0], 2))*y[1] - y[0] ])

# [t, y, h] = solve (dynamic_stiff, 0.0, np.array([2., 0.]), 10.0, 1e-3)

print (len(y))
fig, ax = plt.subplots()
#plt.plot(t, y)
#plt.plot(t, h)

plt.xlabel('time')
plt.ylabel('Solution y')
plt.title("Simple Plot")
plt.legend()
plt.show()
